
from celery import Celery


def get_active_tasks_id(app: Celery):
    ping = app.control.inspect().ping()
    if ping is None:
        return None

    workers = ping.keys()
    i = app.control.inspect(list(workers))
    ids = []

    for key, tasks in i.active().items():
        ids += [task['id'] for task in tasks]

    for key, tasks in i.reserved().items():
        ids += [task['id'] for task in tasks]

    for key, tasks in i.scheduled().items():
        ids += [task['request']['id'] for task in tasks]

    return ids




import re

from django import forms
from django.contrib import admin

__all__ = ['FieldArrayTextFilter']


class FieldArrayTextFilter(admin.FieldListFilter):

    template = 'medicine_utils/admin/array_text_filter.html'

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.lookup_kwarg = '{}__in'.format(field_path)

        for p in self.expected_parameters():
            if p in params:
                value = params.pop(p)
                value = value.replace('\n', ',')
                value = re.sub('[^0-9,]', '', value)
                value = value.strip(',')
                value = ', '.join(list(filter(lambda v: v.isdigit(), [v for v in value.split(',')])))
                params[p] = value

        super().__init__(field, request, params, model, model_admin, field_path)

        self.form = self.get_form(request)

    def get_form(self, request):
        form_class = self._get_form_class()
        form_defaults = {
            self.lookup_kwarg: ','.join(self.used_parameters.get(self.lookup_kwarg, ''))
        }
        return form_class(form_defaults)

    def choices(self, cl):
        yield {
            'system_name': self.lookup_kwarg,
            'query_string': cl.get_query_string(
                {}, remove=self.lookup_kwarg
            )
        }

    def expected_parameters(self):
        return [self.lookup_kwarg]

    def _get_form_class(self):
        fields = {
            self.lookup_kwarg: forms.CharField(widget=forms.Textarea())
        }

        form_class = type(
            str('Form'),
            (forms.BaseForm,),
            {'base_fields': fields}
        )
        return form_class




import xmltodict

from io import BytesIO
from lxml import etree

from django import forms


class XmlDataDictFileField(forms.FileField):

    def to_python(self, data):
        f = super().to_python(data)
        if f is None:
            return None

        if hasattr(data, 'temporary_file_path'):
            file = data
        else:
            if hasattr(data, 'read'):
                file = BytesIO(data.read())
            else:
                file = BytesIO(data['content'])

        try:
            f.tree = etree.parse(file)
            f.data = xmltodict.parse(BytesIO(etree.tostring(f.tree)))
        except etree.XMLSyntaxError:
            raise forms.ValidationError('Неправильный формат файла')

        return f




from django import forms


class AceEditorWidget(forms.Textarea):

    def __init__(self, mode=None, *args, **kwargs):
        self.mode = mode
        super().__init__(*args, **kwargs)

    @property
    def media(self):
        js = [
            'medicine_utils/ace/ace.js',
            'medicine_utils/ace/ext-language_tools.js',
            'medicine_utils/ace/theme-twilight.js'
        ]

        if self.mode:
            js.append('medicine_utils/ace/mode-%s.js' % self.mode)

        css = {'screen': ['medicine_utils/ace/style.css']}

        return forms.Media(js=js, css=css)

    def render(self, name, value, attrs=None, renderer=None):
        attrs = attrs or {}
        id_name = attrs['id'].replace('-', '_')
        textarea = super().render(name, value, attrs, renderer)
        return (f'''
            {textarea}
            <div id="{attrs['id']}_editor" class="ace-editor"></div>
            <script>
            var textarea_{id_name} = django.jQuery("#{attrs['id']}").hide();
            var editor_{id_name} = ace.edit("{attrs['id']}_editor");
            editor_{id_name}.setTheme("ace/theme/twilight");
            editor_{id_name}.session.setMode("ace/mode/{self.mode}");
            editor_{id_name}.setOptions({{
                fontSize: '14px',
                minLines: 10,
                maxLines: Infinity,
                enableBasicAutocompletion: true
            }});
            editor_{id_name}.getSession().setValue(textarea_{id_name}.val());
            editor_{id_name}.getSession().on('change', function(){{
              textarea_{id_name}.val(editor_{id_name}.getSession().getValue());
            }});
            </script> 
            <style>
            .field-{name} {{
                margin: 0 -40px 0 -40px;
                padding: 20px 20px 20px 0;
                background: #232323;
            }}
            label[for="{attrs['id']}"] {{
                display: none;
            }}
            </style>
        ''')




CONSTANCE_ADDITIONAL_FIELDS = {
    'char': ['django.forms.fields.CharField', {
        'widget': 'django.forms.TextInput',
        'widget_kwargs': dict(attrs={'size': 60}),
        'required': False
    }],
    'url': ['django.forms.fields.URLField', {
        'widget': 'django.forms.TextInput',
        'widget_kwargs': dict(attrs={'size': 60}),
        'required': False
    }],
}

CONSTANCE_ADDITIONAL_FIELDS.update(**{
    f'char_length_{n}': ['django.forms.fields.CharField', {
        'min_length': n,
        'max_length': n,
        'required': False,
        'widget': 'django.forms.TextInput',
        'widget_kwargs': dict(attrs={'size': 20})
    }] for n in range(1, 10)
})

CONSTANCE_ADDITIONAL_FIELDS.update(**{
    f'char_length_{n}_required': ['django.forms.fields.CharField', {
        'min_length': n,
        'max_length': n,
        'required': True,
        'widget': 'django.forms.TextInput',
        'widget_kwargs': dict(attrs={'size': 20})
    }] for n in range(1, 10)
})




import datetime

from django import forms
from django.utils.dates import MONTHS


class DateYearMonthWidget(forms.MultiWidget):

    def __init__(self, attrs=None, **kwargs):
        year_start = kwargs.pop('year_start', 2007)
        year_end = kwargs.pop('year_end', datetime.datetime.now().year + 1)
        widgets = (
            forms.Select(choices=list(MONTHS.items())),
            forms.Select(choices=((year, year) for year in range(year_start, year_end))),
        )
        super().__init__(widgets, attrs)

    def decompress(self, value):
        if value:
            return [value.month, value.year]
        return [None, None]


class DateYearMonthField(forms.DateField):

    widget = DateYearMonthWidget

    def to_python(self, value):
        if isinstance(value, (list, tuple)):
            year = value[1]
            month = str(value[0]).zfill(2)
            value = f'{year}-{month}-01'
        return super().to_python(value)



import setuptools
from os.path import join, dirname

import medicine_utils

setuptools.setup(
    name='medicine-utils',
    packages=setuptools.find_packages(),
    include_package_data=True,
    version=medicine_utils.__version__,
    license='MIT',
    url='https://bitbucket.org/med-logic/medicine-utils',
    long_description=open(join(dirname(__file__), 'README.md')).read(),
    long_description_content_type="text/markdown"
)




from pytils.numeral import choose_plural


def convert_to_int(value, default=None):
    if isinstance(value, (int,)):
        return value

    if value.isdigit():
        return int(value)

    return default


def age_plural(age, skip_zero=False, string_format='{age} {age_plural}', default=None):
    age = convert_to_int(age)
    if age is not None:
        if age > 0 or (age == 0 and skip_zero is False):
            return string_format.format(age=age, age_plural=choose_plural(age, ('год', 'года', 'лет')))

    return default


def month_plural(month, string_format='{month} {month_plural}', default=None):
    month = convert_to_int(month)
    if month is not None:
        return string_format.format(month=month, month_plural=choose_plural(month, ('месяц', 'месяца', 'месяцев')))

    return default


def days_plural(days, string_format='{days} {days_plural}', default=None):
    days = convert_to_int(days)
    if days is not None:
        return string_format.format(days=days, days_plural=choose_plural(days, ('день', 'дня', 'дней')))

    return default


def smart_age_plural(age, month, default=None):
    age = convert_to_int(age, default=0)
    month = convert_to_int(month, default=0)
    if age > 0 and month > 0:
        return '{age_plural} {month_plural}'.format(
            age_plural=age_plural(age),
            month_plural=month_plural(month))

    if age == 0 and month > 0:
        return month_plural(month)

    if age > 0 and month == 0:
        return age_plural(age)

    return default


def age_display(years, months, days):
    if years > 0:
        if years < 2 and months > 0:
            return f'{age_plural(years)} {month_plural(months)}'
        else:
            return age_plural(years)
    elif months > 0:
        return month_plural(months)
    else:
        return days_plural(days)
